/***************************************************************
 * Name:      OrdenTreeCtrl.h
 * Purpose:   Defines Dialog OrdenTreeCtrl
 * Author:    Salvador Pozo (salvador@conclase.net)
 * Created:   2025-05-27
 * Copyright: Salvador Pozo (www.conclase.net)
 * License:
 **************************************************************/

#ifndef ORDENTREECTRL_H
#define ORDENTREECTRL_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

//#define wxUSE_TREECTRL TRUE
#include <wx/treectrl.h>
#include <wx/generic/treectlg.h>

class TreeData : public wxTreeItemData {
private:
    int id;
    int idpadre;
    wxString etiqueta;
    int valor;
public:
    TreeData(int i, int p, const wxString etq, int v) : id(i), idpadre(p), etiqueta(etq), valor(v) {}
    int Id() const { return id; }
    int Padre() const { return idpadre; }
    int Valor() const { return valor; }
    wxString Etiqueta() {
        wxString cad;
        cad << etiqueta << " (" << valor << ")";
        return cad;
    }
    wxString &Etq() { return etiqueta; }
    void SetEtiqueta(const wxString &equ) { etiqueta = equ; }
};

class MiTreeCtrl : public wxGenericTreeCtrl {
public:
    MiTreeCtrl(wxWindow *parent, wxWindowID id = wxID_ANY, const wxSize &size = wxDefaultSize, long style = wxTR_DEFAULT_STYLE) :
        wxGenericTreeCtrl(parent, id, wxDefaultPosition, size, style) {}
    virtual int OnCompareItems(const wxTreeItemId &item1, const wxTreeItemId &item2);
};

class OrdenTreeCtrl : public wxDialog
{
    protected:
        enum
        {
            idTreeCtrl = 1000,
            idOrden,
            idBorrar
        };
        MiTreeCtrl* arbol;
        std::vector<TreeData> &elemento;
        wxString &cadena;

    public:
        OrdenTreeCtrl(wxWindow *wnd, std::vector<TreeData> &data, wxString &cad);
        virtual ~OrdenTreeCtrl();
        void OnOk(wxCommandEvent& event);

    private:
        void AsignaIconosRama(const wxTreeItemId &nodo);
        void OrdenarHijos(wxTreeItemId nodo);
        void OnOrdenar(wxCommandEvent& event);
        void OnBorrar(wxCommandEvent& event);
        void OnClose(wxCloseEvent& event);
        void OnDeleteItem(wxTreeEvent& event);

        wxDECLARE_EVENT_TABLE();
};

#endif // ORDENTREECTRL_H
