/***************************************************************
 * Name:      DragDropTreeCtrl.cpp
 * Purpose:   Code for Dialog DragDropTreeCtrl
 * Author:    Salvador Pozo (salvapozo@gmail.com)
 * Created:   2025-05-30
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#include "DragDropTreeCtrl.h"

wxBEGIN_EVENT_TABLE(DragDropTreeCtrl, wxDialog)
    EVT_CLOSE(DragDropTreeCtrl::OnClose)
    EVT_BUTTON(wxID_OK, DragDropTreeCtrl::OnOk)
    EVT_TREE_BEGIN_DRAG(idTreeCtrl, DragDropTreeCtrl::OnBeginDrag)
    EVT_TREE_END_DRAG(idTreeCtrl, DragDropTreeCtrl::OnEndDrag)
wxEND_EVENT_TABLE()

DragDropTreeCtrl::DragDropTreeCtrl(wxWindow *wnd, std::vector<TreeData2> &data, wxString &cad)
    : wxDialog(wnd, -1, _T("wxTreeCtrl Drag&Drop")), datos(data), cadena(cad) {
    wxBoxSizer *boxVertical;

    boxVertical = new wxBoxSizer(wxVERTICAL);

    arbol = new wxGenericTreeCtrl(this, idTreeCtrl, wxDefaultPosition, wxSize(200,200), wxTR_MULTIPLE | wxTR_DEFAULT_STYLE | wxTR_EDIT_LABELS);

    boxVertical->Add(arbol, wxSizerFlags().Expand().Border());

    boxVertical->Add(CreateStdDialogButtonSizer(wxOK | wxCANCEL), wxSizerFlags().Border());

    SetSizerAndFit(boxVertical);

    wxImageList *buts = new wxImageList(16,16);

    buts->Add(wxBitmap("navigate_plus.png", wxBITMAP_TYPE_PNG),  wxColour(255,255,255));
    buts->Add(wxBitmap("navigate_plus.png", wxBITMAP_TYPE_PNG),  wxColour(255,255,255));
    buts->Add(wxBitmap("navigate_minus.png", wxBITMAP_TYPE_PNG), wxColour(255,255,255));
    buts->Add(wxBitmap("navigate_minus.png", wxBITMAP_TYPE_PNG), wxColour(255,255,255));
    arbol->AssignButtonsImageList(buts);

    arbol->AddRoot(_T("Agenda"));
    // Datos de ítem necesarios para mostrar mensajes:
    arbol->SetItemData(arbol->GetRootItem(), new TreeData2(0, 0, "Agenda"));

    InsertarElementos();
}

DragDropTreeCtrl::~DragDropTreeCtrl()
{
    //dtor
}

void DragDropTreeCtrl::InsertarElementos() {
    wxTreeItemId nodo;
    TreeData2* datoElemento;

    for(size_t i=0; i<datos.size(); i++) {
        datoElemento = new TreeData2(datos[i]);
        if(datoElemento->Padre() == 0) {
            nodo = arbol->AppendItem(arbol->GetRootItem(), datos[i].Etiqueta(), 4, 5, datoElemento);
            datoElemento->SetItemId(nodo);
        } else {
            // Buscar nodo padre
            BuscaNodo(datos[i].Padre(), arbol->GetRootItem());
            if(padre.IsOk()) {
                arbol->AppendItem(padre, datos[i].Etiqueta(), 4, 5, datoElemento);
            } else {
                delete datoElemento;
            }
        }
    }
}

void DragDropTreeCtrl::LeerArbol(const wxTreeItemId& idParent, wxTreeItemIdValue cookie)
{
    wxTreeItemId nodo;
    TreeData2* dato;

    if ( !cookie )
        nodo = arbol->GetFirstChild(idParent, cookie);
    else
        nodo = arbol->GetNextChild(idParent, cookie);

    if (!nodo.IsOk()) return;

    dato = static_cast<TreeData2*>(arbol->GetItemData(nodo));
    datos.push_back(*dato);
    if(arbol->ItemHasChildren(nodo)) LeerArbol(nodo);
    LeerArbol(idParent, cookie);
}

// Busca el nodo con el Id indicado
void DragDropTreeCtrl::BuscaNodo(int id, const wxTreeItemId& idParent, wxTreeItemIdValue cookie)
{
    wxTreeItemId nodo;
    TreeData2* dato;

    if ( !cookie )
        nodo = arbol->GetFirstChild(idParent, cookie);
    else
        nodo = arbol->GetNextChild(idParent, cookie);

    if (!nodo.IsOk()) return;

    dato = static_cast<TreeData2*>(arbol->GetItemData(nodo));
    if(dato->Id() != id) {
        if(arbol->ItemHasChildren(nodo)) BuscaNodo(id, nodo);
        BuscaNodo(id, idParent, cookie);
    } else padre = nodo;
}

void DragDropTreeCtrl::OnClose(wxCloseEvent& event) {
    Destroy();
}

void DragDropTreeCtrl::OnOk(wxCommandEvent& event) {
    wxArrayTreeItemIds sels;
    size_t n;
    n = arbol->GetSelections(sels);
    cadena = _T("");
    for(size_t i=0; i<n; i++) {
        cadena += arbol->GetItemText(sels[i]);
        cadena += _T(", ");
    }
    cadena += _T("nº selecciones: ");
    cadena << n;

    // Recuperar datos modificados:
    datos.clear();
    LeerArbol(arbol->GetRootItem());

    EndModal(wxID_OK);
}

void DragDropTreeCtrl::OnBeginDrag(wxTreeEvent& event) {
    event.Allow();
    origen = event.GetItem();
}

void DragDropTreeCtrl::OnEndDrag(wxTreeEvent& event) {
    wxString cad;
    TreeData2* org;
    TreeData2* dest;

    destino = event.GetItem();
    if (!destino.IsOk() || origen==destino) return;
    org  = static_cast<TreeData2*>(arbol->GetItemData(origen));
    dest = static_cast<TreeData2*>(arbol->GetItemData(destino));

    // mover el elemento arrastrado
    cad << org->Etiqueta() << " > " << dest->Etiqueta();
    wxMessageBox(cad, _T("Drag&Drop"));

    // Copia recursiva:
    CopiaHijos(origen, destino);

    // Borra origen
    arbol->Delete(origen);
}

void DragDropTreeCtrl::CopiaHijos(wxTreeItemId desde, wxTreeItemId hasta) {
    wxTreeItemIdValue cookie;
    wxTreeItemId hijo;
    wxTreeItemId nuevoId;
    TreeData2* copiaData;
    TreeData2* dataOrigen;
    TreeData2* dataDestino;

    // Datos de cliente del elemento a mover:
    dataOrigen = static_cast<TreeData2*>(arbol->GetItemData(desde));
    // Datos de cliente del elemento de destino:
    dataDestino = static_cast<TreeData2*>(arbol->GetItemData(hasta));
    // Creamos una copia de esos datos:
    copiaData = new TreeData2(*dataOrigen);
    // Insertamos una copia del elemento en la nueva ubicación:
    nuevoId = arbol->AppendItem(hasta, copiaData->Etiqueta(), 4, 5, copiaData);
    // Actualizamos el identificador del elemento en los datos de cliente:
    copiaData->SetPadre(dataDestino->Id());

    if(arbol->ItemHasChildren(desde)) {
        hijo = arbol->GetFirstChild(desde, cookie);
        do {
            CopiaHijos(hijo, nuevoId);
            hijo = arbol->GetNextChild(hijo, cookie);
        } while(hijo.IsOk());
    }
}
