/***************************************************************
 * Name:      EditTreeCtrl.cpp
 * Purpose:   Code for Dialog EditTreeCtrl
 * Author:    Salvador Pozo (salvapozo@gmail.com)
 * Created:   2025-05-29
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#include "EditTreeCtrl.h"

wxBEGIN_EVENT_TABLE(EditTreeCtrl, wxDialog)
    EVT_CLOSE(EditTreeCtrl::OnClose)
    EVT_BUTTON(wxID_OK, EditTreeCtrl::OnOk)
    EVT_TREE_BEGIN_LABEL_EDIT(idTreeCtrl, EditTreeCtrl::OnBeginEditLabel)
    EVT_TREE_END_LABEL_EDIT(idTreeCtrl, EditTreeCtrl::OnEndEditLabel)

    EVT_TREE_BEGIN_DRAG(idTreeCtrl, EditTreeCtrl::OnBeginDrag)
    EVT_TREE_END_DRAG(idTreeCtrl, EditTreeCtrl::OnEndDrag)
wxEND_EVENT_TABLE()

EditTreeCtrl::EditTreeCtrl(wxWindow *wnd, std::vector<TreeData2> &data, wxString &cad)
    : wxDialog(wnd, -1, _T("wxTreeCtrl editable")), datos(data), cadena(cad) {
    wxBoxSizer *boxVertical;

    boxVertical = new wxBoxSizer(wxVERTICAL);

    arbol = new wxGenericTreeCtrl(this, idTreeCtrl, wxDefaultPosition, wxSize(200,200), wxTR_MULTIPLE | wxTR_DEFAULT_STYLE | wxTR_HIDE_ROOT | wxTR_EDIT_LABELS);

    boxVertical->Add(arbol, wxSizerFlags().Expand().Border());

    boxVertical->Add(CreateStdDialogButtonSizer(wxOK | wxCANCEL), wxSizerFlags().Border());

    SetSizerAndFit(boxVertical);

    wxImageList *buts = new wxImageList(16,16);

    buts->Add(wxBitmap("navigate_plus.png", wxBITMAP_TYPE_PNG),  wxColour(255,255,255));
    buts->Add(wxBitmap("navigate_plus.png", wxBITMAP_TYPE_PNG),  wxColour(255,255,255));
    buts->Add(wxBitmap("navigate_minus.png", wxBITMAP_TYPE_PNG), wxColour(255,255,255));
    buts->Add(wxBitmap("navigate_minus.png", wxBITMAP_TYPE_PNG), wxColour(255,255,255));
    arbol->AssignButtonsImageList(buts);

    arbol->AddRoot(_T("Raíz"));

    // Copiar los datos a dataItem:
    for(size_t i=0; i<data.size(); i++) {
        dataItem.push_back(new TreeData2(data[i]));
    }
    InsertarElementos();
}

EditTreeCtrl::~EditTreeCtrl()
{
    //dtor
}

void EditTreeCtrl::InsertarElementos() {
    wxTreeItemId nodo;

    for(size_t i=0; i<dataItem.size(); i++) {
        if(dataItem[i]->Padre() == 0) {
            nodo = arbol->AppendItem(arbol->GetRootItem(), dataItem[i]->Etiqueta(), 4, 5, dataItem[i]);
            dataItem[i]->SetItemId(nodo);
        } else {
            // Buscar nodo padre
            for(size_t j=0; j<i; j++) {
                if(dataItem[i]->Padre() == dataItem[j]->Id()) {
                    if(dataItem[j]->GetItemId().IsOk()) {
                        nodo = arbol->AppendItem(dataItem[j]->GetItemId(), dataItem[i]->Etiqueta(), 4, 5, dataItem[i]);
                        dataItem[i]->SetItemId(nodo);
                    }
                }
            }
        }
    }
}

void EditTreeCtrl::OnClose(wxCloseEvent& event) {
    Destroy();
}

void EditTreeCtrl::OnOk(wxCommandEvent& event) {
    wxArrayTreeItemIds sels;
    size_t n;
    n = arbol->GetSelections(sels);
    cadena = _T("");
    for(size_t i=0; i<n; i++) {
        cadena += arbol->GetItemText(sels[i]);
        cadena += _T(", ");
    }
    cadena += _T("nº selecciones: ");
    cadena << n;

    // Recuperar datos modificados:
    for(size_t i=0; i<datos.size(); i++) {
        datos[i] = *dataItem[i];
    }
    EndModal(wxID_OK);
}

void EditTreeCtrl::OnBeginEditLabel(wxTreeEvent& event) {
    wxTreeItemId itemId = event.GetItem();
    TreeData2* data = static_cast<TreeData2*>(arbol->GetItemData(itemId));
    arbol->SetItemText(itemId, data->Etiqueta());
}

void EditTreeCtrl::OnEndEditLabel(wxTreeEvent& event) {
    wxTreeItemId itemId = event.GetItem();
    TreeData2* data = static_cast<TreeData2*>(arbol->GetItemData(itemId));
    if(event.IsEditCancelled()) {
        arbol->SetItemText(itemId, data->Etiqueta());
        return;
    }
    data->SetEtiqueta(event.GetLabel());
    arbol->SetItemText(itemId, data->Etiqueta());
}

void EditTreeCtrl::OnBeginDrag(wxTreeEvent& event) {
    event.Allow();
}

void EditTreeCtrl::OnEndDrag(wxTreeEvent& event) {
    // mover el elemento arrastrado
}
