/***************************************************************
 * Name:      OrdenTreeCtrl.cpp
 * Purpose:   Code for Dialog OrdenTreeCtrl
 * Author:    Salvador Pozo (salvapozo@gmail.com)
 * Created:   2025-05-27
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#include "OrdenTreeCtrl.h"
#include "left_blue.xpm"
#include "left_green.xpm"
#include "down_blue.xpm"
#include "down_green.xpm"
#include "blue.xpm"
#include "green.xpm"

int MiTreeCtrl::OnCompareItems(const wxTreeItemId &item1, const wxTreeItemId &item2) {
    TreeData *dato1 = static_cast<TreeData*>(GetItemData(item1));
    TreeData *dato2 = static_cast<TreeData*>(GetItemData(item2));
    return dato1->Valor()-dato2->Valor();
}

wxBEGIN_EVENT_TABLE(OrdenTreeCtrl, wxDialog)
    EVT_CLOSE(OrdenTreeCtrl::OnClose)
    EVT_BUTTON(idOrden, OrdenTreeCtrl::OnOrdenar)
    EVT_BUTTON(idBorrar, OrdenTreeCtrl::OnBorrar)
    EVT_BUTTON(wxID_OK, OrdenTreeCtrl::OnOk)
    EVT_TREE_DELETE_ITEM(idTreeCtrl, OrdenTreeCtrl::OnDeleteItem)
wxEND_EVENT_TABLE()

OrdenTreeCtrl::OrdenTreeCtrl(wxWindow *wnd, std::vector<TreeData> &data, wxString &cad)
    : wxDialog(wnd, -1, _T("wxTreeCtrl ordenado")), elemento(data), cadena(cad) {
    wxBoxSizer *boxVertical;
    wxBoxSizer *boxHorizontal;
    wxTreeItemId nodo;
    wxTreeItemIdValue cookie;
    TreeData *dato;

    boxVertical = new wxBoxSizer(wxVERTICAL);
    boxHorizontal = new wxBoxSizer(wxHORIZONTAL);

    arbol = new MiTreeCtrl(this, idTreeCtrl, wxSize(200,200), wxTR_MULTIPLE | wxTR_DEFAULT_STYLE | wxTR_HIDE_ROOT);

    boxVertical->Add(arbol, wxSizerFlags().Expand().Border());

    wxButton *boton = new wxButton(this, idOrden, "Ordenar");
    boxHorizontal->Add(boton, wxSizerFlags().Border());
    boton = new wxButton(this, idBorrar, "Eliminar");
    boxHorizontal->Add(boton, wxSizerFlags().Border());
    boxVertical->Add(boxHorizontal, wxSizerFlags().Border());

    boxVertical->Add(CreateStdDialogButtonSizer(wxOK | wxCANCEL), wxSizerFlags().Border());

    SetSizerAndFit(boxVertical);

    wxVector<wxBitmapBundle> imgs;
    imgs.push_back(wxBitmapBundle(down_blue_xpm));
    imgs.push_back(wxBitmapBundle(down_green_xpm));
    imgs.push_back(wxBitmapBundle(left_blue_xpm));
    imgs.push_back(wxBitmapBundle(left_green_xpm));
    imgs.push_back(wxBitmapBundle(blue_xpm));
    imgs.push_back(wxBitmapBundle(green_xpm));
    arbol->SetImages(imgs);

    wxImageList *buts = new wxImageList(16,16);

    buts->Add(wxBitmap("navigate_plus.png", wxBITMAP_TYPE_PNG),  wxColour(255,255,255));
    buts->Add(wxBitmap("navigate_plus.png", wxBITMAP_TYPE_PNG),  wxColour(255,255,255));
    buts->Add(wxBitmap("navigate_minus.png", wxBITMAP_TYPE_PNG), wxColour(255,255,255));
    buts->Add(wxBitmap("navigate_minus.png", wxBITMAP_TYPE_PNG), wxColour(255,255,255));
    arbol->AssignButtonsImageList(buts);

    arbol->AddRoot(_T("Raíz"));

    for(size_t i=0; i<elemento.size(); i++) {
        TreeData* treeData = new TreeData(elemento[i]);
        if(elemento[i].Padre() == 0) {
            nodo = arbol->AppendItem(arbol->GetRootItem(), elemento[i].Etiqueta(),-1,-1, treeData);
            AsignaIconosRama(nodo);
        } else {
            // Buscar nodo padre
            nodo = arbol->GetFirstChild(arbol->GetRootItem(), cookie);
            do {
                dato = static_cast<TreeData*>(arbol->GetItemData(nodo));
                if(dato->Id() != elemento[i].Padre()) {
                    nodo = arbol->GetNextSibling(nodo);
                    dato = static_cast<TreeData*>(arbol->GetItemData(nodo));
                }
            } while(nodo.IsOk() && dato->Id() != elemento[i].Padre());
            if(nodo.IsOk()) arbol->AppendItem(nodo, elemento[i].Etiqueta(), 4, 5, treeData);
        }
    }
}

OrdenTreeCtrl::~OrdenTreeCtrl()
{
    //dtor
}
void OrdenTreeCtrl::AsignaIconosRama(const wxTreeItemId &nodo) {
    arbol->SetItemImage(nodo, 0, wxTreeItemIcon_Normal);
    arbol->SetItemImage(nodo, 1, wxTreeItemIcon_Selected);
    arbol->SetItemImage(nodo, 2, wxTreeItemIcon_Expanded);
    arbol->SetItemImage(nodo, 3, wxTreeItemIcon_SelectedExpanded);
}

void OrdenTreeCtrl::OnClose(wxCloseEvent& event) {
    Destroy();
}

void OrdenTreeCtrl::OnOrdenar(wxCommandEvent& event) {
    OrdenarHijos(arbol->GetRootItem());
}

void OrdenTreeCtrl::OrdenarHijos(wxTreeItemId nodo) {
    wxTreeItemIdValue cookie;

    arbol->SortChildren(nodo);

    wxTreeItemId nodo2 = arbol->GetFirstChild(nodo, cookie);
    do {
        if(nodo2.IsOk()) {
            OrdenarHijos(nodo2);
            nodo2 = arbol->GetNextChild(nodo, cookie);
        }
    } while(nodo2.IsOk());
}

void OrdenTreeCtrl::OnBorrar(wxCommandEvent& event) {
    if(arbol->GetFocusedItem().IsOk())
        arbol->Delete(arbol->GetFocusedItem());
}

void OrdenTreeCtrl::OnOk(wxCommandEvent& event) {
    wxArrayTreeItemIds sels;
    size_t n;
    n = arbol->GetSelections(sels);
    cadena = _T("");
    for(size_t i=0; i<n; i++) {
        cadena += arbol->GetItemText(sels[i]);
        cadena += _T(", ");
    }
    cadena += _T("nº selecciones: ");
    cadena << n;
    EndModal(wxID_OK);
}

void OrdenTreeCtrl::OnDeleteItem(wxTreeEvent& event) {
    wxTreeItemId itemId = event.GetItem();
    TreeData* data = static_cast<TreeData*>(arbol->GetItemData(itemId));
    wxString msg;
    msg << _T("Elemento '") << data->Etq() << _T("' eliminado.");
    wxMessageBox(msg, _T("Borrar elemento"));
}
