/***************************************************************
 * Name:      TreeCtrl.cpp
 * Purpose:   Code for Dialog TreeCtrl
 * Author:    Salvador Pozo (salvapozo@gmail.com)
 * Created:   2025-05-26
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#include "TreeCtrl.h"
#include "left_blue.xpm"
#include "left_green.xpm"
#include "down_blue.xpm"
#include "down_green.xpm"
#include "blue.xpm"
#include "green.xpm"

wxBEGIN_EVENT_TABLE(TreeCtrl, wxDialog)
    EVT_CLOSE(TreeCtrl::OnClose)
    EVT_BUTTON(wxID_OK, TreeCtrl::OnOk)
wxEND_EVENT_TABLE()

TreeCtrl::TreeCtrl(wxWindow *wnd, wxString &cad)
    : wxDialog(wnd, -1, _T("wxTreeCtrl")), cadena(cad) {
    wxBoxSizer *boxVertical;
    wxTreeItemId nodo;
    wxTreeItemId nodo2;

    boxVertical = new wxBoxSizer(wxVERTICAL);

    arbol = new wxTreeCtrl(this, idTreeCtrl, wxDefaultPosition, wxSize(200,200), wxTR_MULTIPLE | wxTR_DEFAULT_STYLE | wxTR_HIDE_ROOT);

    boxVertical->Add(arbol, wxSizerFlags().Expand().Border());

    boxVertical->Add(CreateStdDialogButtonSizer(wxOK | wxCANCEL), wxSizerFlags().Border());
    SetSizerAndFit(boxVertical);

    wxVector<wxBitmapBundle> imgs;
    imgs.push_back(wxBitmapBundle(down_blue_xpm));
    imgs.push_back(wxBitmapBundle(down_green_xpm));
    imgs.push_back(wxBitmapBundle(left_blue_xpm));
    imgs.push_back(wxBitmapBundle(left_green_xpm));
    imgs.push_back(wxBitmapBundle(blue_xpm));
    imgs.push_back(wxBitmapBundle(green_xpm));
    arbol->SetImages(imgs);

    arbol->AddRoot(_T("Raíz"));

    nodo = arbol->AppendItem(arbol->GetRootItem(), _T("Hijo 1"), 4, 5);
    nodo = arbol->AppendItem(arbol->GetRootItem(), _T("Hijo 3"), 4, 5);
    nodo2 = arbol->InsertItem(arbol->GetRootItem(), 1, _T("Hijo 2"));
    AsignaIconosRama(nodo2);
    nodo = arbol->AppendItem(arbol->GetRootItem(), _T("Hijo 4"), 4, 5);

    nodo = arbol->AppendItem(nodo2, _T("Hijo 2-1"), 4, 5);
    nodo = arbol->AppendItem(nodo2, _T("Hijo 2-3"), 4, 5);
    nodo = arbol->AppendItem(nodo2, _T("Hijo 2-2"), 4, 5);
    nodo = arbol->AppendItem(nodo2, _T("Hijo 2-4"), 4, 5);
    nodo = arbol->AppendItem(nodo2, _T("Hijo 2-5"), 4, 5);

    arbol->SortChildren(nodo2);
    //arbol->SetItemState(nodo4, 3);
}

TreeCtrl::~TreeCtrl()
{
    //dtor
}
void TreeCtrl::AsignaIconosRama(const wxTreeItemId &nodo) {
    arbol->SetItemImage(nodo, 0, wxTreeItemIcon_Normal);
    arbol->SetItemImage(nodo, 1, wxTreeItemIcon_Selected);
    arbol->SetItemImage(nodo, 2, wxTreeItemIcon_Expanded);
    arbol->SetItemImage(nodo, 3, wxTreeItemIcon_SelectedExpanded);
}

void TreeCtrl::OnClose(wxCloseEvent& event) {
    Destroy();
}

void TreeCtrl::OnOk(wxCommandEvent& event) {
    wxArrayTreeItemIds sels;
    size_t n;
    n = arbol->GetSelections(sels);
    cadena = _T("");
    for(size_t i=0; i<n; i++) {
        cadena += arbol->GetItemText(sels[i]);
        cadena += _T(",");
    }
    cadena += _T("nº selecciones: ");
    cadena << n;
    EndModal(wxID_OK);
}
