/***************************************************************
 * Name:      wx022Main.cpp
 * Purpose:   Code for Application Frame
 * Author:    Salvador Pozo (salvador@conclase.net)
 * Created:   2025-05-23
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#include <wx/imagpng.h>
#include "wx022Main.h"

//helper functions
enum wxbuildinfoformat {
    short_f, long_f };

wxString wxbuildinfo(wxbuildinfoformat format)
{
    wxString wxbuild(wxVERSION_STRING);

    if (format == long_f )
    {
#if defined(__WXMSW__)
        wxbuild << _T("-Windows");
#elif defined(__WXMAC__)
        wxbuild << _T("-Mac");
#elif defined(__UNIX__)
        wxbuild << _T("-Linux");
#endif

#if wxUSE_UNICODE
        wxbuild << _T("-Unicode build");
#else
        wxbuild << _T("-ANSI build");
#endif // wxUSE_UNICODE
    }

    return wxbuild;
}

BEGIN_EVENT_TABLE(wx022Frame, wxFrame)
    EVT_CLOSE(wx022Frame::OnClose)
    EVT_MENU(idMenuTreeCtrl, wx022Frame::OnTreeCtrl)
    EVT_MENU(idMenuGenericTree, wx022Frame::OnGenericTreeCtrl)
    EVT_MENU(idMenuOrdenTree, wx022Frame::OnOrdenTreeCtrl)
    EVT_MENU(idMenuEditTree, wx022Frame::OnEditTreeCtrl)
    EVT_MENU(idMenuDragDropTree, wx022Frame::OnDragDropTreeCtrl)
    EVT_MENU(idMenuQuit, wx022Frame::OnQuit)
    EVT_MENU(idMenuAbout, wx022Frame::OnAbout)
END_EVENT_TABLE()

wx022Frame::wx022Frame(wxFrame *frame, const wxString& title)
    : wxFrame(frame, -1, title)
{
    // Datos para el ejemplo de árbol ordenado
    datos.push_back(TreeData(1, 0, _T("Manuel"), 34));
    datos.push_back(TreeData(2, 1, _T("José Luis"), 17));
    datos.push_back(TreeData(3, 1, _T("Mariana"), 25));
    datos.push_back(TreeData(4, 0, _T("Dolores"), 74));
    datos.push_back(TreeData(5, 4, _T("Felipe"), 48));
    datos.push_back(TreeData(6, 4, _T("Ana"), 50));
    datos.push_back(TreeData(8, 0, _T("Alejandro"), 63));
    datos.push_back(TreeData(7, 4, _T("Luisa"), 49));
    datos.push_back(TreeData(9, 8, _T("Gabriel"), 15));
    datos.push_back(TreeData(10, 8, _T("Sofia"), 18));

    // Datos para el ejemplo de árbol editable
    datos2.push_back(TreeData2(1, 0, _T("Manuel")));
    datos2.push_back(TreeData2(2, 1, _T("Teléfono")));
    datos2.push_back(TreeData2(3, 2, _T("Casa")));
    datos2.push_back(TreeData2(4, 3, _T("555 22 33 44")));
    datos2.push_back(TreeData2(5, 2, _T("Trabajo")));
    datos2.push_back(TreeData2(6, 5, _T("555 77 88 99")));
    datos2.push_back(TreeData2(7, 0, _T("Julia")));
    datos2.push_back(TreeData2(8, 7, _T("email")));
    datos2.push_back(TreeData2(9, 8, _T("julia@ejemplo.com")));
    datos2.push_back(TreeData2(10, 7, _T("Teléfono")));
    datos2.push_back(TreeData2(11, 10, _T("Móvil")));
    datos2.push_back(TreeData2(12, 11, _T("555 88 99 88")));
    datos2.push_back(TreeData2(13, 0, _T("Timoteo")));
    datos2.push_back(TreeData2(14, 0, _T("Penélope")));
    datos2.push_back(TreeData2(15, 14, _T("email")));
    datos2.push_back(TreeData2(16, 15, _T("penelope@ejemplo.net")));
    datos2.push_back(TreeData2(17, 13, _T("Teléfono")));
    datos2.push_back(TreeData2(18, 17, _T("555 44 22 33")));


    wxImage::AddHandler(new wxPNGHandler);
    // create a menu bar
    wxMenuBar* mbar = new wxMenuBar();
    wxMenu* fileMenu = new wxMenu(_T(""));
    fileMenu->Append(idMenuTreeCtrl, _T("&TreeCtrl"), _T("Diálogo TreeCtrl"));
    fileMenu->Append(idMenuGenericTree, _T("&TreeCtrl genérico"), _T("Diálogo GenericTreeCtrl"));
    fileMenu->Append(idMenuOrdenTree, _T("&TreeCtrl ordenado"), _T("Diálogo OrdenTreeCtrl"));
    fileMenu->Append(idMenuEditTree, _T("&TreeCtrl etitar etiquetas"), _T("Diálogo OrdenTreeCtrl"));
    fileMenu->Append(idMenuDragDropTree, _T("&TreeCtrl arrastrar etiquetas"), _T("Diálogo DragDropTreeCtrl"));
    fileMenu->Append(idMenuQuit, _T("&Salir\tAlt-F4"), _T("Salir de la aplicación"));
    mbar->Append(fileMenu, _("&Fichero"));

    wxMenu* helpMenu = new wxMenu(_T(""));
    helpMenu->Append(idMenuAbout, _T("&Sobre\tF1"), _T("Muestra información sobre la aplicación"));
    mbar->Append(helpMenu, _T("&Ayuda"));

    SetMenuBar(mbar);

    // create a status bar with some information about the used wxWidgets version
    CreateStatusBar(2);
    SetStatusText(_T("Hola"),0);
    SetStatusText(wxbuildinfo(short_f), 1);
}


wx022Frame::~wx022Frame()
{
}

void wx022Frame::OnClose(wxCloseEvent &event)
{
    Destroy();
}

void wx022Frame::OnQuit(wxCommandEvent &event)
{
    Destroy();
}

void wx022Frame::OnAbout(wxCommandEvent &event)
{
    wxString msg = wxbuildinfo(long_f);
    wxMessageBox(msg, _("Bienvenido a..."));
}

void wx022Frame::OnTreeCtrl(wxCommandEvent& event)
{
    wxString cadena;
    TreeCtrl *dlg = new TreeCtrl(this, cadena);
    if(dlg->ShowModal() == wxID_OK) {
        wxMessageBox(cadena, _T("TreeCtrl"));
    }
}

void wx022Frame::OnGenericTreeCtrl(wxCommandEvent& event)
{
    wxString cadena;
    GenericTreeCtrl *dlg = new GenericTreeCtrl(this, cadena);
    if(dlg->ShowModal() == wxID_OK) {
        wxMessageBox(cadena, _T("TreeCtrl"));
    }
}

void wx022Frame::OnOrdenTreeCtrl(wxCommandEvent& event)
{
    wxString cadena;
    OrdenTreeCtrl *dlg = new OrdenTreeCtrl(this, datos, cadena);
    if(dlg->ShowModal() == wxID_OK) {
        wxMessageBox(cadena, _T("TreeCtrl"));
    }
}

void wx022Frame::OnEditTreeCtrl(wxCommandEvent& event)
{
    wxString cadena;
    EditTreeCtrl *dlg = new EditTreeCtrl(this, datos2, cadena);
    if(dlg->ShowModal() == wxID_OK) {
        wxMessageBox(cadena, _T("TreeCtrl"));
    }
}

void wx022Frame::OnDragDropTreeCtrl(wxCommandEvent& event)
{
    wxString cadena;
    DragDropTreeCtrl *dlg = new DragDropTreeCtrl(this, datos2, cadena);
    if(dlg->ShowModal() == wxID_OK) {
        wxMessageBox(cadena, _T("TreeCtrl"));
    }
}
